@@ -344,6 +344,21 @@ public class ConnectionTest extends Common {
     }
   }
 
+  @Test
+  public void databaseNoStateChange() throws SQLException {
+    try (Connection connection = createCon("&disableSessionTracking=true")) {
+      assertEquals(database, connection.getCatalog());
+
+      try (Statement stmt = connection.createStatement()) {
+        stmt.execute("drop database if exists _test_db");
+        stmt.execute("create database _test_db");
+        stmt.execute("USE _test_db");
+        assertEquals("_test_db", connection.getCatalog());
+        stmt.execute("drop database _test_db");
+      }
+    }
+  }
+
   @Test
   public void catalog() throws SQLException {
     Assumptions.assumeTrue(
@@ -383,7 +398,8 @@ public class ConnectionTest extends Common {
     assertTrue(sharedConn.createClob() instanceof Clob);
     assertTrue(sharedConn.createNClob() instanceof NClob);
     assertThrows(SQLException.class, () -> sharedConn.createSQLXML());
-    assertThrows(SQLException.class, () -> sharedConn.createArrayOf("", null));
+    assertNull(sharedConn.createArrayOf("", null));
+    assertThrows(SQLException.class, () -> sharedConn.createArrayOf("string", "ddd"));
     assertThrows(SQLException.class, () -> sharedConn.createStruct("", null));
     assertNull(sharedConn.getSchema());
     sharedConn.setSchema("fff");
@@ -578,7 +594,6 @@ public class ConnectionTest extends Common {
       Statement stmt = connection.createStatement();
       boolean canUseTransactionReadOnly =
           (isMariaDBServer()
-                  && sharedConn.getContext().getVersion().getMajorVersion() < 23
                   && sharedConn.getContext().getVersion().versionGreaterOrEqual(11, 1, 1))
               || (!sharedConn.getContext().getVersion().isMariaDBServer()
                   && ((sharedConn.getContext().getVersion().getMajorVersion() >= 8
@@ -760,6 +775,54 @@ public class ConnectionTest extends Common {
     stmt.execute("drop user verificationEd25519AuthPlugin@'%'");
   }
 
+  @Test
+  public void parsecAuthPlugin() throws Throwable {
+    Assumptions.assumeTrue(
+        !"maxscale".equals(System.getenv("srv")) && isMariaDBServer() && minVersion(10, 6, 1));
+    Statement stmt = sharedConn.createStatement();
+
+    try {
+      stmt.execute("INSTALL SONAME 'auth_parsec'");
+    } catch (SQLException sqle) {
+      Assumptions.assumeTrue(false, "server doesn't have auth_parsec plugin, cancelling test");
+    }
+
+    stmt.execute("drop user IF EXISTS verifParsec@'%'");
+    stmt.execute("drop user IF EXISTS verifParsec2@'%'");
+    stmt.execute(
+        "CREATE USER verifParsec@'%' IDENTIFIED VIA parsec USING PASSWORD('MySup8%rPassw@ord')");
+    stmt.execute("CREATE USER verifParsec2@'%' IDENTIFIED VIA parsec USING PASSWORD('')");
+    stmt.execute("GRANT SELECT on `" + database + "`.* to verifParsec@'%'");
+    stmt.execute("GRANT SELECT on `" + database + "`.* to verifParsec2@'%'");
+
+    String version = System.getProperty("java.version");
+    int majorVersion =
+        (version.indexOf(".") >= 0)
+            ? Integer.parseInt(version.substring(0, version.indexOf(".")))
+            : Integer.parseInt(version);
+    if (majorVersion < 15) {
+      // before java 15, Ed25519 is not supported
+      // assuming, that BouncyCastle is not on test classpath
+      assertThrowsContains(
+          SQLException.class,
+          () -> createCon("user=verifParsec&password=MySup8%rPassw@ord"),
+          "Parsec authentication not available. Either use Java 15+ or add BouncyCastle"
+              + " dependency");
+    } else {
+      try (Connection connection = createCon("user=verifParsec&password=MySup8%rPassw@ord")) {
+        // must have succeeded
+        connection.getCatalog();
+      }
+    }
+
+    assertThrowsContains(
+        SQLException.class,
+        () -> createCon("user=verifParsec2&password=MySup8%rPassw@ord&restrictedAuth=dialog"),
+        "Client restrict authentication plugin to a limited set");
+    stmt.execute("drop user verifParsec@'%'");
+    stmt.execute("drop user verifParsec2@'%'");
+  }
+
   @Test
   public void pamAuthPlugin() throws Throwable {
     // https://mariadb.com/kb/en/authentication-plugin-pam/
@@ -1180,7 +1243,7 @@ public class ConnectionTest extends Common {
     // ensure connecting without DB
     String connStr =
         String.format(
-            "jdbc:mariadb://%s:%s/?user=%s&password=%s&%s&createDatabaseIfNotExist&allowPublicKeyRetrieval=true",
+            "jdbc:mariadb://%s:%s/?user=%s&password=%s&%s&createDatabaseIfNotExist",
             hostname, port, user, password, defaultOther);
     try (Connection con = DriverManager.getConnection(connStr)) {
       con.createStatement().executeQuery("SELECT 1");
@@ -1189,7 +1252,7 @@ public class ConnectionTest extends Common {
     String nonExistentDatabase = "bla`f`l";
     connStr =
         String.format(
-            "jdbc:mariadb://%s:%s/%s?user=%s&password=%s&%s&createDatabaseIfNotExist&allowPublicKeyRetrieval=true",
+            "jdbc:mariadb://%s:%s/%s?user=%s&password=%s&%s&createDatabaseIfNotExist",
             hostname, port, nonExistentDatabase, user, password, defaultOther);
     try (Connection con = DriverManager.getConnection(connStr)) {
       ResultSet rs = con.createStatement().executeQuery("select DATABASE()");
@@ -1200,7 +1263,7 @@ public class ConnectionTest extends Common {
     nonExistentDatabase = "bla`f`l0";
     connStr =
         String.format(
-            "jdbc:mariadb:replication://%s:%s,%s:%s/%s?user=%s&password=%s&%s&createDatabaseIfNotExist&allowPublicKeyRetrieval=true",
+            "jdbc:mariadb:replication://%s:%s,%s:%s/%s?user=%s&password=%s&%s&createDatabaseIfNotExist",
             hostname, port, hostname, port, nonExistentDatabase, user, password, defaultOther);
     try (Connection con = DriverManager.getConnection(connStr)) {
       ResultSet rs = con.createStatement().executeQuery("select DATABASE()");
@@ -1371,25 +1434,4 @@ public class ConnectionTest extends Common {
       }
     }
   }
-
-  @Test
-  public void isClosed() throws SQLException {
-    Connection con = DriverManager.getConnection(mDefUrl);
-    Statement stmt = con.createStatement();
-    PreparedStatement preparedStatement = con.prepareStatement("SELECT ?");
-    assertFalse(stmt.isClosed());
-    assertFalse(preparedStatement.isClosed());
-
-    stmt.execute("SELECT 1");
-    preparedStatement.setInt(1, 1);
-    preparedStatement.execute();
-
-    assertFalse(stmt.isClosed());
-    assertFalse(preparedStatement.isClosed());
-
-    con.close();
-
-    assertTrue(stmt.isClosed());
-    assertTrue(preparedStatement.isClosed());
-  }
 }
