@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: LGPL-2.1-or-later
 // Copyright (c) 2012-2014 Monty Program Ab
-// Copyright (c) 2015-2023 MariaDB Corporation Ab
+// Copyright (c) 2015-2024 MariaDB Corporation Ab
 package org.mariadb.jdbc.integration;
 
 import static org.junit.jupiter.api.Assertions.*;
@@ -12,7 +12,10 @@ import java.math.RoundingMode;
 import java.net.URL;
 import java.nio.charset.StandardCharsets;
 import java.sql.*;
+import java.time.LocalDateTime;
+import java.time.ZonedDateTime;
 import java.util.Calendar;
+import java.util.Map;
 import java.util.TimeZone;
 import org.junit.jupiter.api.AfterAll;
 import org.junit.jupiter.api.Assumptions;
@@ -150,14 +153,14 @@ public class PreparedStatementParametersTest extends Common {
         rs -> assertEquals(Date.valueOf("2010-05-25"), rs.getDate(1)),
         con);
     boolean minus = TimeZone.getDefault().getOffset(System.currentTimeMillis()) > 0;
+
+    ZonedDateTime zdt =
+        LocalDateTime.parse((minus ? "2010-01-11" : "2010-01-12") + "T00:00:00.0")
+            .atZone(TimeZone.getTimeZone("UTC").toZoneId());
+
     checkSendTimestamp(
         ps -> ps.setDate(1, Date.valueOf("2010-01-12"), utcCal),
-        rs ->
-            assertEquals(minus ? 1263164400000L : 1263254400000L, rs.getDate(1, utcCal).getTime()),
-        con);
-    checkSendTimestamp(
-        ps -> ps.setDate(1, Date.valueOf("2010-01-12"), utcCal),
-        rs -> assertEquals(minus ? "2010-01-11" : "2010-01-12", rs.getDate(1, utcCal).toString()),
+        rs -> assertEquals(zdt.toEpochSecond() * 1000, rs.getDate(1, utcCal).getTime()),
         con);
     checkSendTimestamp(
         ps -> ps.setDate(1, Date.valueOf("2010-05-25")),
@@ -310,6 +313,7 @@ public class PreparedStatementParametersTest extends Common {
         ps -> ps.setURL(1, new URL("https://www.someUrl.com")),
         rs -> assertEquals("https://www.someUrl.com", rs.getString(1)),
         con);
+    checkSendString(ps -> ps.setURL(1, (URL) null), rs -> assertNull(rs.getString(1)), con);
     // TODO SET OBJECT
   }
 
@@ -353,8 +357,75 @@ public class PreparedStatementParametersTest extends Common {
           "REF parameter are not supported");
       Common.assertThrowsContains(
           SQLException.class,
-          () -> preparedStatement.setArray(1, null),
-          "Array parameter are not supported");
+          () -> preparedStatement.setObject(1, "", Types.REF),
+          "Type not supported");
+      preparedStatement.setArray(1, null);
+      Common.assertThrowsContains(
+          SQLException.class,
+          () -> con.createArrayOf("String", new Float[] {1f}),
+          "typeName String is not supported");
+      Common.assertThrowsContains(
+          SQLException.class,
+          () ->
+              preparedStatement.setArray(
+                  1,
+                  new Array() {
+                    @Override
+                    public String getBaseTypeName() throws SQLException {
+                      return null;
+                    }
+
+                    @Override
+                    public int getBaseType() throws SQLException {
+                      return 0;
+                    }
+
+                    @Override
+                    public Object getArray() throws SQLException {
+                      return null;
+                    }
+
+                    @Override
+                    public Object getArray(Map<String, Class<?>> map) throws SQLException {
+                      return null;
+                    }
+
+                    @Override
+                    public Object getArray(long index, int count) throws SQLException {
+                      return null;
+                    }
+
+                    @Override
+                    public Object getArray(long index, int count, Map<String, Class<?>> map)
+                        throws SQLException {
+                      return null;
+                    }
+
+                    @Override
+                    public ResultSet getResultSet() throws SQLException {
+                      return null;
+                    }
+
+                    @Override
+                    public ResultSet getResultSet(Map<String, Class<?>> map) throws SQLException {
+                      return null;
+                    }
+
+                    @Override
+                    public ResultSet getResultSet(long index, int count) throws SQLException {
+                      return null;
+                    }
+
+                    @Override
+                    public ResultSet getResultSet(long index, int count, Map<String, Class<?>> map)
+                        throws SQLException {
+                      return null;
+                    }
+
+                    @Override
+                    public void free() throws SQLException {}
+                  }),
+          "this type of Array parameter");
       Common.assertThrowsContains(
           SQLException.class,
           () -> preparedStatement.setRowId(1, null),
