@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: LGPL-2.1-or-later
 // Copyright (c) 2012-2014 Monty Program Ab
-// Copyright (c) 2015-2023 MariaDB Corporation Ab
+// Copyright (c) 2015-2024 MariaDB Corporation Ab
 package org.mariadb.jdbc.integration;
 
 import static org.junit.jupiter.api.Assertions.*;
@@ -25,6 +25,7 @@ public class PreparedStatementTest extends Common {
     stmt.execute("DROP TABLE IF EXISTS prepare2");
     stmt.execute("DROP TABLE IF EXISTS prepare3");
     stmt.execute("DROP TABLE IF EXISTS prepare4");
+    stmt.execute("DROP TABLE IF EXISTS prepare5");
   }
 
   @BeforeAll
@@ -36,6 +37,7 @@ public class PreparedStatementTest extends Common {
     stmt.execute("CREATE TABLE prepare3 (t1 LONGTEXT, t2 LONGTEXT, t3 LONGTEXT, t4 LONGTEXT)");
     stmt.execute("CREATE TABLE prepare4 (t1 int)");
     stmt.execute("INSERT INTO prepare4 VALUES (1),(2),(3),(4),(5)");
+    stmt.execute("CREATE TABLE prepare5 (t1 int)");
     createSequenceTables();
   }
 
@@ -50,6 +52,15 @@ public class PreparedStatementTest extends Common {
         .toString();
   }
 
+  @Test
+  public void prepToString() throws SQLException {
+    try (PreparedStatement stmt = sharedConn.prepareStatement("SELECT ?,?")) {
+      stmt.setNull(2, Types.INTEGER);
+      Assertions.assertEquals(
+          "ClientPreparedStatement{sql:'SELECT ?,?', parameters:[null,null]}", stmt.toString());
+    }
+  }
+
   @Test
   public void prep() throws SQLException {
     try (PreparedStatement stmt = sharedConn.prepareStatement("SELECT ?")) {
@@ -235,7 +246,6 @@ public class PreparedStatementTest extends Common {
       preparedStatement.setInt(2, 10);
       assertEquals(1, preparedStatement.executeUpdate());
 
-      // verification that query without resultset return an empty resultset
       preparedStatement.clearParameters();
       Common.assertThrowsContains(
           SQLException.class,
@@ -243,8 +253,7 @@ public class PreparedStatementTest extends Common {
           "Parameter at position 1 is not set");
       preparedStatement.setInt(2, 11);
       preparedStatement.setInt(1, 6);
-      ResultSet rs0 = preparedStatement.executeQuery();
-      assertFalse(rs0.next());
+      preparedStatement.executeUpdate();
 
       // verification
       ResultSet rs = stmt.executeQuery("SELECT * FROM prepare1");
@@ -597,28 +606,50 @@ public class PreparedStatementTest extends Common {
       preparedStatement.addBatch();
       preparedStatement.setInt(1, 20);
       preparedStatement.addBatch();
+      preparedStatement.setInt(1, 40);
+      preparedStatement.addBatch();
       preparedStatement.executeBatch();
       ResultSet rs = preparedStatement.getGeneratedKeys();
       assertTrue(rs.next());
       assertEquals(1, rs.getInt(1));
       assertTrue(rs.next());
       assertEquals(2, rs.getInt(1));
+      assertTrue(rs.next());
+      assertEquals(3, rs.getInt(1));
       assertFalse(rs.next());
     }
 
     try (PreparedStatement preparedStatement =
         sharedConnBinary.prepareStatement(
             "INSERT INTO prepare2(t2) VALUES (?)", java.sql.Statement.RETURN_GENERATED_KEYS)) {
+
+      preparedStatement.setInt(1, 0);
+      preparedStatement.executeUpdate();
+      ResultSet rs = preparedStatement.getGeneratedKeys();
+      assertTrue(rs.next());
+      assertEquals(4, rs.getInt(1));
+
       preparedStatement.setInt(1, 10);
       preparedStatement.addBatch();
       preparedStatement.setInt(1, 20);
       preparedStatement.addBatch();
+      preparedStatement.setInt(1, 50);
+      preparedStatement.addBatch();
       preparedStatement.executeBatch();
-      ResultSet rs = preparedStatement.getGeneratedKeys();
+      rs = preparedStatement.getGeneratedKeys();
       assertTrue(rs.next());
-      assertEquals(3, rs.getInt(1));
+      assertEquals(5, rs.getInt(1));
       assertTrue(rs.next());
-      assertEquals(4, rs.getInt(1));
+      assertEquals(6, rs.getInt(1));
+      assertTrue(rs.next());
+      assertEquals(7, rs.getInt(1));
+      assertFalse(rs.next());
+
+      preparedStatement.setInt(1, 50);
+      preparedStatement.execute();
+      rs = preparedStatement.getGeneratedKeys();
+      assertTrue(rs.next());
+      assertEquals(8, rs.getInt(1));
       assertFalse(rs.next());
     }
 
@@ -632,9 +663,9 @@ public class PreparedStatementTest extends Common {
         preparedStatement.execute();
         ResultSet rs = preparedStatement.getGeneratedKeys();
         assertTrue(rs.next());
-        assertEquals(5, rs.getInt(1));
+        assertEquals(9, rs.getInt(1));
         assertTrue(rs.next());
-        assertEquals(6, rs.getInt(1));
+        assertEquals(10, rs.getInt(1));
         assertFalse(rs.next());
 
         preparedStatement.setInt(1, 210);
@@ -647,13 +678,13 @@ public class PreparedStatementTest extends Common {
 
         rs = preparedStatement.getGeneratedKeys();
         assertTrue(rs.next());
-        assertEquals(7, rs.getInt(1));
+        assertEquals(11, rs.getInt(1));
         assertTrue(rs.next());
-        assertEquals(8, rs.getInt(1));
+        assertEquals(12, rs.getInt(1));
         assertTrue(rs.next());
-        assertEquals(9, rs.getInt(1));
+        assertEquals(13, rs.getInt(1));
         assertTrue(rs.next());
-        assertEquals(10, rs.getInt(1));
+        assertEquals(14, rs.getInt(1));
         assertFalse(rs.next());
       }
     }
@@ -1090,6 +1121,50 @@ public class PreparedStatementTest extends Common {
     }
   }
 
+  @Test
+  public void decrementCacheBatch() throws SQLException {
+    decrementCacheBatch("&useServerPrepStmts=false&prepStmtCacheSize=5");
+    decrementCacheBatch("&useServerPrepStmts=true&prepStmtCacheSize=5");
+    decrementCacheBatch("&useServerPrepStmts=true&useServerPrepStmts=false");
+  }
+
+  private void decrementCacheBatch(String connString) throws SQLException {
+    try (Connection con = createCon(connString)) {
+      int i = 0;
+      PreparedStatement prep = con.prepareStatement("INSERT INTO prepare5 VALUES (?)");
+      prep.setInt(1, i++);
+      prep.addBatch();
+      prep.setInt(1, i++);
+      prep.addBatch();
+      prep.executeBatch();
+
+      PreparedStatement prep2 = con.prepareStatement("INSERT INTO prepare5 VALUES (?)");
+      prep2.setInt(1, i++);
+      prep2.addBatch();
+      prep2.setInt(1, i++);
+      prep2.addBatch();
+      prep2.executeBatch();
+
+      for (int ii = 1; ii < 10; ii++) {
+        try (PreparedStatement prep3 =
+            con.prepareStatement("INSERT INTO prepare5 VALUES (?) /*" + ii + "*/")) {
+          prep3.setInt(1, i++);
+          prep3.addBatch();
+          prep3.setInt(1, i++);
+          prep3.addBatch();
+          prep3.executeBatch();
+          prep3.setQueryTimeout(1);
+        }
+      }
+
+      prep.setQueryTimeout(1); // will close prepare
+      prep2.setQueryTimeout(2); // will close prepare
+      prep.execute();
+      prep.close();
+      prep2.close();
+    }
+  }
+
   @Test
   public void prepareStatementConcur() throws SQLException {
     try (Connection con = createCon("&useServerPrepStmts=false")) {
@@ -1292,6 +1367,70 @@ public class PreparedStatementTest extends Common {
     }
   }
 
+  @Test
+  public void closeOnCompletionClose() throws Exception {
+    closeOnCompletionClose(sharedConn);
+    closeOnCompletionClose(sharedConnBinary);
+  }
+
+  @Test
+  public void closeOnCompletionStreamingClose() throws Exception {
+    try (Connection conn = createCon("&allowMultiQueries")) {
+      try (PreparedStatement pstmt = conn.prepareStatement("SELECT 1;SELECT 2")) {
+        pstmt.execute();
+        pstmt.closeOnCompletion();
+        assertTrue(pstmt.getMoreResults(org.mariadb.jdbc.Statement.CLOSE_CURRENT_RESULT));
+        assertFalse(pstmt.getMoreResults(java.sql.Statement.CLOSE_CURRENT_RESULT));
+        assertThrows(SQLException.class, () -> pstmt.execute());
+      }
+
+      try (PreparedStatement pstmt =
+          conn.prepareStatement("SELECT * from sequence_1_to_10;SELECT * from sequence_1_to_10")) {
+        pstmt.execute();
+        pstmt.closeOnCompletion();
+        assertTrue(pstmt.getMoreResults(org.mariadb.jdbc.Statement.CLOSE_CURRENT_RESULT));
+        try (ResultSet rs = pstmt.getResultSet()) {
+          assertTrue(rs.next());
+        }
+        assertThrows(SQLException.class, () -> pstmt.execute());
+      }
+      try (PreparedStatement pstmt = conn.prepareStatement("SELECT 1;SELECT 2")) {
+        pstmt.execute();
+        pstmt.closeOnCompletion();
+        assertTrue(pstmt.getMoreResults(org.mariadb.jdbc.Statement.CLOSE_CURRENT_RESULT));
+        assertFalse(pstmt.getMoreResults(java.sql.Statement.CLOSE_CURRENT_RESULT));
+        assertThrows(SQLException.class, () -> pstmt.execute());
+      }
+
+      try (PreparedStatement pstmt = conn.prepareStatement("SELECT 1;SELECT 2")) {
+        pstmt.setFetchSize(1);
+        pstmt.execute();
+        pstmt.closeOnCompletion();
+        assertTrue(pstmt.getMoreResults(org.mariadb.jdbc.Statement.CLOSE_CURRENT_RESULT));
+        try (ResultSet rs = pstmt.getResultSet()) {
+          assertTrue(rs.next());
+        }
+        assertThrows(SQLException.class, () -> pstmt.execute());
+      }
+    }
+  }
+
+  private void closeOnCompletionClose(Connection connection) throws Exception {
+    try (PreparedStatement pstmt = connection.prepareStatement("SELECT 1;")) {
+      pstmt.execute();
+      pstmt.closeOnCompletion();
+      assertFalse(pstmt.getMoreResults(Statement.CLOSE_CURRENT_RESULT));
+      assertThrows(SQLSyntaxErrorException.class, () -> pstmt.execute());
+    }
+    try (PreparedStatement pstmt = connection.prepareStatement("SELECT 1;")) {
+      pstmt.setFetchSize(1);
+      pstmt.execute();
+      pstmt.closeOnCompletion();
+      assertFalse(pstmt.getMoreResults(Statement.CLOSE_CURRENT_RESULT));
+      assertThrows(SQLSyntaxErrorException.class, () -> pstmt.execute());
+    }
+  }
+
   @Test
   public void textPrefix() throws SQLException {
     try (Connection con = createCon("&useServerPrepStmts&allowMultiQueries")) {
@@ -1299,7 +1438,7 @@ public class PreparedStatementTest extends Common {
       try (PreparedStatement prep =
           con.prepareStatement("/*client prepare*/SET @name := ?; SELECT @name ")) {
         prep.setString(1, "test");
-        prep.executeQuery();
+        prep.execute();
         assertTrue(prep.getMoreResults(Statement.CLOSE_CURRENT_RESULT));
         ResultSet rs = prep.getResultSet();
         assertTrue(rs.next());
@@ -1307,4 +1446,129 @@ public class PreparedStatementTest extends Common {
       }
     }
   }
+
+  @Test
+  public void getUpdateCountValueOnFail() throws SQLException {
+    getUpdateCountValueOnFail(sharedConn);
+    getUpdateCountValueOnFail(sharedConnBinary);
+  }
+
+  private void getUpdateCountValueOnFail(Connection conn) throws SQLException {
+    try (Statement st = conn.createStatement()) {
+      st.execute("DROP TABLE IF EXISTS getUpdateCountValueOnFail");
+      try (PreparedStatement prep =
+          conn.prepareStatement(
+              "CREATE TABLE getUpdateCountValueOnFail(id VARCHAR(5) PRIMARY KEY,value BOOL)")) {
+        assertEquals(-1, prep.getUpdateCount());
+        assertEquals(0, prep.executeUpdate());
+        assertEquals(0, prep.getUpdateCount());
+        try {
+          prep.executeUpdate();
+        } catch (Exception e) {
+          // eat
+        }
+        assertEquals(-1, prep.getUpdateCount());
+      } finally {
+        st.execute("DROP TABLE IF EXISTS getUpdateCountValueOnFail");
+      }
+    }
+  }
+
+  @Test
+  public void testBatchParameterClearAfterError() throws SQLException {
+    testBatchParameterClearAfterError(sharedConn);
+    testBatchParameterClearAfterError(sharedConnBinary);
+  }
+
+  public void testBatchParameterClearAfterError(Connection conn) throws SQLException {
+    try (Statement stmt = conn.createStatement()) {
+      stmt.execute("DROP TABLE IF EXISTS testBatchParameterClearAfterError");
+      stmt.execute(
+          "CREATE TABLE testBatchParameterClearAfterError(id TINYINT PRIMARY KEY,value SMALLINT)");
+      try (PreparedStatement prep =
+          conn.prepareStatement("INSERT INTO testBatchParameterClearAfterError VALUES(1, 1)")) {
+        prep.setInt(1, 1);
+        prep.setInt(2, 1);
+        prep.addBatch();
+        prep.setInt(1, 1);
+        prep.setInt(2, 1);
+        prep.addBatch();
+        assertThrows(BatchUpdateException.class, prep::executeBatch);
+        stmt.execute("TRUNCATE testBatchParameterClearAfterError");
+        prep.executeBatch();
+      }
+
+      try (ResultSet rs = stmt.executeQuery("SELECT * FROM testBatchParameterClearAfterError")) {
+        assertFalse(rs.next());
+      }
+    }
+  }
+
+  @Test
+  public void testLargeBatchParameterClearAfterError() throws SQLException {
+    testLargeBatchParameterClearAfterError(sharedConn);
+    testLargeBatchParameterClearAfterError(sharedConnBinary);
+  }
+
+  public void testLargeBatchParameterClearAfterError(Connection conn) throws SQLException {
+    try (Statement stmt = conn.createStatement()) {
+      stmt.execute("DROP TABLE IF EXISTS testBatchParameterClearAfterError");
+      stmt.execute(
+          "CREATE TABLE testBatchParameterClearAfterError(id TINYINT PRIMARY KEY,value SMALLINT)");
+      try (PreparedStatement prep =
+          conn.prepareStatement("INSERT INTO testBatchParameterClearAfterError VALUES(1, 1)")) {
+        prep.setInt(1, 1);
+        prep.setInt(2, 1);
+        prep.addBatch();
+        prep.setInt(1, 1);
+        prep.setInt(2, 1);
+        prep.addBatch();
+        assertThrows(BatchUpdateException.class, prep::executeLargeBatch);
+        stmt.execute("TRUNCATE testBatchParameterClearAfterError");
+        prep.executeLargeBatch();
+      }
+
+      try (ResultSet rs = stmt.executeQuery("SELECT * FROM testBatchParameterClearAfterError")) {
+        assertFalse(rs.next());
+      }
+    }
+  }
+
+  @Test
+  public void prepCacheMultiDb() throws SQLException {
+    Statement stmt = sharedConn.createStatement();
+    stmt.execute("DROP DATABASE IF EXISTS dbtest1");
+    stmt.execute("DROP DATABASE IF EXISTS dbtest2");
+    stmt.execute("CREATE DATABASE dbtest1");
+    stmt.execute("CREATE DATABASE dbtest2");
+    stmt.execute("CREATE TABLE dbtest1.commontable(t1 varchar(32))");
+    stmt.execute("CREATE TABLE dbtest2.commontable(t2 varchar(32))");
+
+    stmt.execute("INSERT INTO dbtest1.commontable VALUES ('db1')");
+    stmt.execute("INSERT INTO dbtest2.commontable VALUES ('db2')");
+    try (Connection con = createCon("&cachePrepStmts&useServerPrepStmts")) {
+      con.setCatalog("dbtest1");
+      try (PreparedStatement prep = con.prepareStatement("SELECT * FROM commontable")) {
+        ResultSet rs = prep.executeQuery();
+        rs.next();
+        assertEquals("db1", rs.getString(1));
+      }
+      try (PreparedStatement prep = con.prepareStatement("SELECT * FROM commontable")) {
+        ResultSet rs = prep.executeQuery();
+        rs.next();
+        assertEquals("db1", rs.getString(1));
+      }
+      con.setCatalog("dbtest2");
+      try (PreparedStatement prep = con.prepareStatement("SELECT * FROM commontable")) {
+        ResultSet rs = prep.executeQuery();
+        rs.next();
+        assertEquals("db2", rs.getString(1));
+      }
+      try (PreparedStatement prep = con.prepareStatement("SELECT * FROM commontable")) {
+        ResultSet rs = prep.executeQuery();
+        rs.next();
+        assertEquals("db2", rs.getString(1));
+      }
+    }
+  }
 }
