@@ -903,11 +903,11 @@ public class IntCodecTest extends CommonCodecTest {
 
   private void getArray(ResultSet rs) {
     Common.assertThrowsContains(
-        SQLException.class, () -> rs.getArray(1), "Method ResultSet.getArray not supported");
+        SQLException.class, () -> rs.getArray(1), "Data type INTEGER cannot be decoded as float[]");
     Common.assertThrowsContains(
         SQLException.class,
         () -> rs.getArray("t1alias"),
-        "Method ResultSet.getArray not supported");
+        "Data type INTEGER cannot be decoded as float[]");
   }
 
   @Test
